/*
*************************************************************************************************
*		CM50012ACT6 65k color Graphical LCD Display Driver (HYVIX HDC1600 controller)
*										All Rights Reserved
* File name		: hdc1600.c
* Programmer 	: John Leung, TechToys Co. Hong Kong
* Web presence  : www.TechToys.com.hk
* Note			: 
* Language		: C18 complier version 2.40, MPLAB v7.60
* Hardware		: PIC18LF4550-STK1 Rev3b
* Date			: 11 June 2007
* version 		: 1.0a
*************************************************************************************************
*										DESCRIPTION
*
* This module provides an interface to CM50012ACT6 65k color graphical LCD module of 128x160dots
* Display controller chip is Hyvix HDC1600
* Module requires 3.3V DC supply to Vdd, LED backlight requires 6.2-6.5V, 15mA typical
*
* LCD pinout function summarized as below
* --------- LCD		MCU -----------------
* pin 1		LED-K 	- GND (Cathode for LED-backlight)
* pin 2 	LED+A 	- LED backlight, requires 6.2-6.5V, 15mA typical
* pin 3		Vss  	- GND
* pin 4		Vdd		- 3.3V Supply
* pin 5		n.c.	- shunt to ground for PIC18LF4550-STK1
* pin 6		n.c.	- shunt to ground for PIC18LF4550-STK1
* pin 7		CS		- chip select, low active
* pin 8		RST		- reset, low active; wired to manual reset switch of the whole system
* pin 9		A0		- (H:Data, L:Command (data or command control))
* pin 10	WR		- active low, L:Write; use with RD pin
* pin 11	RD		- active low, L:Read; use with WR pin
* pin 12:19	D7:D0	- LCD_DATA 
* pin 20			- Vss
***********************************************************************************************
*/


#include <p18cxxx.h>					
#include "hdc1600.h"
#include "sysfont.h"	

/*
*********************************************************************************************************
*											LOCAL DEFINITIONS
*********************************************************************************************************
*/

#define OUTPUT							0
#define INPUT							1
#define HIGH							1
#define LOW								0

#define OSCILLATION_MODE_SET			0x02
#define DRIVER_OUTPUT_MODE_SET			0x10
#define BOOST_COEF_SET					0x22
#define DCDC_CLK_FREQ_SET				0x24	//DC-DC Clock division set
#define DCDC_AMP_ON_OFF_SET				0x26
#define TEMP_COMPENSATION_SET			0x28
#define CONTRAST_CONTROL3				0x29
#define CONTRAST_CONTROL1				0x2A
#define CONTRAST_CONTROL2				0x2B
#define STANDBY_MODE_OFF				0x2C
#define STANDBY_MODE_ON					0x2D
#define DDRAM_BURST_MODE_OFF			0x2E
#define DDRAM_BURST_MODE_ON				0x2F
#define N_LINE_INVERSION_SET			0x34
#define FRAME_FREQUENCY_CONTROL_SET		0x36
#define ENTRY_MODE_SET					0x40
#define X_ADDRESS_AREA_SET				0x42
#define Y_ADDRESS_AREA_SET				0x43
#define CURRENT_BIAS_SET				0x46
#define DISPLAY_OFF						0x50
#define DISPLAY_ON						0x51
#define SPEC_DISPLAY_PATTERN_SET		0x53
#define PARTIAL_DISPLAY_MODE_SET		0x55
#define PARTIAL_DISPLAY_1_START_LINE	0x56
#define PARTIAL_DISPLAY_1_END_LINE		0x57
#define SCROLL_START_1_MEM_SET			0x5A
#define MPE_MODE_OFF					0x5C
#define MPE_MODE_ON						0x5D
#define PARTIAL_DISPLAY_2_START_LINE	0x60
#define PARTIAL_DISPLAY_2_END_LINE		0x62
#define SCROLL_START_2_MEM_SET			0x64
#define COMMON_SCAN_MODE_SET			0x66
#define COMMON_SCAN_MODE_OFF			0x67
#define Y_ADDRESS_DEC_SET				0x68
#define Y_ADDRESS_DEC_OFF				0x69
#define BLANK_TIME_SET					0x6A
#define RED_PALETTE_SET					0x70
#define	GREEN_PALETTE_SET				0x71
#define BLUE_PALETTE_SET				0x72

/*
*********************************************************************************************************
*										LOCAL FUNCTIONS PROTOTYPE
*********************************************************************************************************
*/
static void cDispRstArea(void);


/*
*********************************************************************************************************
*                         			RESET DISPLAY AREA TO 128*160 MATRIX
*
* Description : This function resets the display area to 128*160 matrix, local function
* Arguments   : none
*			
* Returns     : none
* Note		  : This function is a local function.
*				Dependency on cDispWrCmd(DRIVER_OUTPUT_MODE_SET) under cDispInit() because
*				display direction changed on SDIR, and CDR=0 (by hardware).
*********************************************************************************************************
*/
static void cDispRstArea(void)
{
	cDispWrCmd(X_ADDRESS_AREA_SET);			//X-address counter set, offset from 0~159 for 160 dots
	cDispWrCmd(0);
	cDispWrCmd(159);

	cDispWrCmd(Y_ADDRESS_AREA_SET);			//Y-address counter set, offset from 0~127 for 128 dots
	cDispWrCmd(0);
	cDispWrCmd(127);
}

/*
*********************************************************************************************************
*                         				DISPLAY INITIALIZATION
*
* Description : This function initializes the LCD module		
* Arguments   : none
*			
* Returns     : none
* Note		  : This function should be called once before any of the other functions
*********************************************************************************************************
*/
void cDispInit(void)
{
	INT8U p;

	cDispInitPort();

	cDispWrCmd(STANDBY_MODE_OFF);			//Standby mode off

	cDispWrCmd(DCDC_CLK_FREQ_SET);			//DC-DC clock freq. set 12kHz
	cDispWrCmd(0x03);

	cDispWrCmd(FRAME_FREQUENCY_CONTROL_SET);//Main clock frequency set (90Hz)
	cDispWrCmd(0x12);						//1A:1.2+(0.025*1AH) = 1.85MHz
											//Frame rate :1/(128*160*(1/1.85M))=90Hz
	cDispWrCmd(OSCILLATION_MODE_SET);		//oscillator mode set
	cDispWrCmd(0x05);						//Internal DC-DC clock input
											//internal DCDC osc. ON, internal main clock input, internal osc ON
	
	cDispSetQuali(BRIGHTNESS, CONTRAST);

	cDispWrCmd(CURRENT_BIAS_SET);			
	cDispWrCmd(0x07);

	cDispWrCmd(BOOST_COEF_SET);
	cDispWrCmd(0x06);

	cDispWrCmd(TEMP_COMPENSATION_SET);		//set the temperature compensation ratio
	cDispWrCmd(0x00);	

	cDispWrCmd(DRIVER_OUTPUT_MODE_SET);		//SDIR=0(Y-address 00H to 7FH), SWP=0 (normal state, D7-D0 or D15-D0 written
	cDispWrCmd(0x20);						//to the RAM as it is the initial state

	cDispWrCmd(DDRAM_BURST_MODE_ON);		//burst mode ON for data RAM write											//DLN[1:0]=10 => 1/60 display duty

	cDispWrCmd(ENTRY_MODE_SET);				//entry mode set, non-reverse, X address count mode, automatic address increment
	cDispWrCmd(0x00);

	cDispWrCmd(COMMON_SCAN_MODE_SET);

	cDispWrCmd(DCDC_AMP_ON_OFF_SET);
	cDispWrCmd(0x0F);

	cDispRstArea();	

	cDispWrCmd(SPEC_DISPLAY_PATTERN_SET);	//normal display
	cDispWrCmd(0x00);

	cDispWrCmd(PARTIAL_DISPLAY_MODE_SET);	//partial display normal disabled
	cDispWrCmd(0x00);

	cDispWrCmd(MPE_MODE_OFF);

	cDispWrCmd(Y_ADDRESS_DEC_SET);
	cDispWrCmd(0x00);

	cDispWrCmd(BLANK_TIME_SET);
	cDispWrCmd(0x00);
	cDispWrCmd(0x00);


 // RED
    p = PALETTE_RED;
	cDispWrCmd(RED_PALETTE_SET);
	cDispWrCmd(0);
	cDispWrCmd(10+p);
	cDispWrCmd(18+p);
	cDispWrCmd(24+p);
	cDispWrCmd(28+p);
	cDispWrCmd(32+p);
	cDispWrCmd(35+p);
	cDispWrCmd(38+p);
	cDispWrCmd(41+p);
	cDispWrCmd(44+p);

	cDispWrCmd(47+p);
	cDispWrCmd(50+p);
	cDispWrCmd(53+p);
	cDispWrCmd(56+p);
	cDispWrCmd(59+p);
	cDispWrCmd(62+p);
	cDispWrCmd(65+p);
	cDispWrCmd(68+p);
	cDispWrCmd(71+p);
	cDispWrCmd(74+p);

	cDispWrCmd(77+p);
	cDispWrCmd(80+p);
	cDispWrCmd(83+p);
	cDispWrCmd(86+p);
	cDispWrCmd(89+p);
	cDispWrCmd(92+p);
	cDispWrCmd(95+p);
	cDispWrCmd(99+p);
	cDispWrCmd(103+p);
	cDispWrCmd(109+p);

	cDispWrCmd(117+p);
	cDispWrCmd(127);

// BLUE
	p = PALETTE_BLUE;
	cDispWrCmd(BLUE_PALETTE_SET);
	cDispWrCmd(0);
	cDispWrCmd(10+p);
	cDispWrCmd(18+p);
	cDispWrCmd(24+p);
	cDispWrCmd(28+p);
	cDispWrCmd(32+p);
	cDispWrCmd(35+p);
	cDispWrCmd(38+p);
	cDispWrCmd(41+p);
	cDispWrCmd(44+p);

	cDispWrCmd(47+p);
	cDispWrCmd(50+p);
	cDispWrCmd(53+p);
	cDispWrCmd(56+p);
	cDispWrCmd(59+p);
	cDispWrCmd(62+p);
	cDispWrCmd(65+p);
	cDispWrCmd(68+p);
	cDispWrCmd(71+p);
	cDispWrCmd(74+p);

	cDispWrCmd(77+p);
	cDispWrCmd(80+p);
	cDispWrCmd(83+p);
	cDispWrCmd(86+p);
	cDispWrCmd(89+p);
	cDispWrCmd(92+p);
	cDispWrCmd(95+p);
	cDispWrCmd(99+p);
	cDispWrCmd(103+p);
	cDispWrCmd(109+p);

	cDispWrCmd(117+p);
	cDispWrCmd(127);

// GREEN
	p = PALETTE_GREEN;
	cDispWrCmd(GREEN_PALETTE_SET);
	cDispWrCmd(0);
	cDispWrCmd(5+p);
	cDispWrCmd(10+p);
	cDispWrCmd(14+p);
	cDispWrCmd(18+p);
	cDispWrCmd(21+p);
	cDispWrCmd(24+p);
	cDispWrCmd(26+p);
	cDispWrCmd(28+p);
	cDispWrCmd(30+p);

	cDispWrCmd(32+p);
	cDispWrCmd(34+p);
	cDispWrCmd(35+p);
	cDispWrCmd(37+p);
	cDispWrCmd(38+p);
	cDispWrCmd(40+p);
	cDispWrCmd(41+p);
	cDispWrCmd(43+p);
	cDispWrCmd(44+p);
	cDispWrCmd(46+p);

	cDispWrCmd(47+p);
	cDispWrCmd(49+p);
	cDispWrCmd(50+p);
	cDispWrCmd(52+p);
	cDispWrCmd(53+p);
	cDispWrCmd(55+p);
	cDispWrCmd(56+p);
	cDispWrCmd(58+p);
	cDispWrCmd(59+p);
	cDispWrCmd(61+p);

	cDispWrCmd(62+p);
	cDispWrCmd(63+p);
	cDispWrCmd(64+p);
	cDispWrCmd(65+p);
	cDispWrCmd(67+p);
	cDispWrCmd(68+p);
	cDispWrCmd(70+p);
	cDispWrCmd(71+p);
	cDispWrCmd(73+p);
	cDispWrCmd(74+p);

	cDispWrCmd(76+p);
	cDispWrCmd(77+p);
	cDispWrCmd(79+p);
	cDispWrCmd(80+p);
	cDispWrCmd(82+p);
	cDispWrCmd(83+p);
	cDispWrCmd(85+p);
	cDispWrCmd(86+p);
	cDispWrCmd(88+p);
	cDispWrCmd(89+p);

	cDispWrCmd(91+p);
	cDispWrCmd(92+p);
	cDispWrCmd(94+p);
	cDispWrCmd(95+p);
	cDispWrCmd(97+p);
	cDispWrCmd(99+p);
	cDispWrCmd(101+p);
	cDispWrCmd(103+p);
	cDispWrCmd(106+p);
	cDispWrCmd(109+p);
	cDispWrCmd(113+p);
	cDispWrCmd(117+p);
	cDispWrCmd(121+p); 
	cDispWrCmd(127);

	cDispWrCmd(DISPLAY_ON);	
}


/*
*********************************************************************************************************
*                         				DISPLAY ON/OFF SWITCH
*
* Description : This function turns display ON/OFF by writing the command DISPLAY_ON / DISPLAY_OFF
* Arguments   : (BOOLEAN) 'sw' 		ENABLE for turning ON defined under s6b33bc.h
*									DISABLE for turning OFF defined under s6b33bc.h
*				
* Returns     : none
* Notes		  : Internal status and display RAM data not changed by switching between ON and OFF
*********************************************************************************************************
*/
void cDispSwitch(BOOLEAN sw)
{
	(sw==ENABLE)? cDispWrCmd(DISPLAY_ON):cDispWrCmd(DISPLAY_OFF);
}

/*
*********************************************************************************************************
*                         	CLEAR DISPLAY BY WRITING PURE COLOR TO THE WHOLE SCREEN
*
* Description : This function writes a fixed int to all pixels making use of the automatic
*				address increment feature for X and Y address counters
*				
* Arguments   : 'color' is in 16-bit, [R4R3R2R1R0][G5G4G3G2G1G0][B4B3B2B1B0] 
*						Writing color=0xFFFF equivalent to white color
*						color=0x0000 eq. black
*						color=0xF800 eq. red
*						color=0x07E0 eq. green
*						color=0x001F eq. blue
*				
* Returns     : none
*********************************************************************************************************
*/
void cDispClr(INT16U color)
{
	INT8U x,y,hiByte,lowByte;

	hiByte  = color>>8;
	lowByte = (INT8U)color;

		for(x=0;x<MAX_ROW_PIXEL;x++)
		{
			for(y=0;y<MAX_COL_PIXEL;y++)
			{
			cDispWrDat(hiByte);
			cDispWrDat(lowByte);
			}
		}
}


/*
*********************************************************************************************************
*                         				SET PIXEL AT x, y POSITION
*
* Description : This function sets a pixel with (INT16U)color at a position defined by (x,y) following 
*				Cartesian coordinate system. Coordinates (0,0) at the top left corner, 
*				coordinates (127,159) at the lower right corner of the LCD screen, w/ module viewed from
*				frontal up direction on PIC18LF-4550-STK1 Rev3b with LCD controller HDC1600
*
* Arguments   : 'x'    		0....MAX_COL_PIXEL-1 is the matrix position in x-axis (left ->right)
*				'y'    		0....MAX_ROW_PIXEL-1 is the matrix position in y-axis (up -> down)
*				'color'  	sets 16-bit color
* Returns     : none
* Notes		  : Should apply the local function cDispRstArea() to reset display area 128*160
*********************************************************************************************************
*/
void cDispSetPixel(INT8U x, INT8U y, INT16U color)
{
	INT8U hiByte, lowByte;
	x = MAX_COL_PIXEL-1-x;
	y = MAX_ROW_PIXEL-1-y;

	hiByte  = color>>8;
	lowByte = (INT8U)color;

	cDispWrCmd(X_ADDRESS_AREA_SET);		//X-address area set in the 160 pixel direction (y-axis)
	cDispWrCmd(y);						//offset for scan COM from 0~159
	cDispWrCmd(y++);
	cDispWrCmd(Y_ADDRESS_AREA_SET);		//Y-address area set in the 128 pixel direction (x-axis)
	cDispWrCmd(x);						//offset for scan SEG from 0~127
	cDispWrCmd(x++);

	cDispWrDat(hiByte);
	cDispWrDat(lowByte);	

	cDispRstArea();
}

/*
*********************************************************************************************************
*               		CONVERT A STRING FROM ROM TO PIXEL DATA AND DISPLAY AT x,y
*
* Description : This function outputs a string in graphic mode (pixel-by-pixel) at a position defined 
*				by (x,y) following Cartesian coordinate system with variable font spacing defined by 
*				fontSpace. Character wrapping is allowed. String from ROM so const rom keyword used
*				*** REQUIRE AT LEAST ONE FONT SET (sysfont.h) included under this file ****
*				Cartesian coordinates as (0,0) at the top left hand corner, (127,159) at the 
*				lower right hand corner, as seen from frontal up view of the PIC18-LF4550-STK1 board
*
* Arguments   : 'x'    		0....MAX_COL_PIXEL-1 is matrix position in horizontal direction (x-axis).
*							
*				'y'    		0....MAX_ROW_PIXEL-1 is matrix position in vertical direction (y-axis).
*
*				'*pStr'		pointer to an ASCII string in ROM (flash), null terminated
*				'fontSpace' font spacing from 1 to 127
*				'color'		16-bit color possible
* Returns     : none
* Notes		  : Microchip C18 complier dependent as const rom keyword involved.
* Example 	  : ....
*				....
*				main()
*				{ 
*					cDispPixStrAt(0,20,"Hello!",1,RGB(31,0,0)); 	//Display the ASCII string "Hello!"
*																	//at (x,y)=(0,20), font space = 1
*																	//color in red
*				}
*********************************************************************************************************
*/
void cDisprStrAt(INT8U x, INT8U y, const rom char *pStr, INT8U fontSpace, INT16U color)
{
	INT8U pixelByte,bit_idx, i;				//loop counters 
	INT8U c,fwidth, fheight;				//character, font width & height store

	while (*pStr)
	{
		c = *pStr++ - 32;
				
		fwidth = SYS_FNT[c].fontWidth;
		fheight = 8;

		if((x+fwidth)>MAX_COL_PIXEL) 		//character wrapping here				
		{
			x=0;
			y+=fheight;
		}

		if((y+fheight)>MAX_ROW_PIXEL)		//check for y boundary				
		{
			y=0;
		}

		for(i=0;i<fheight;i++)
		{
			pixelByte = SYS_FNT[c].fontBody[i];
			for(bit_idx=0; bit_idx<fwidth; bit_idx++)
				{
						if((pixelByte)&(0x80>>bit_idx))
							cDispSetPixel(x+bit_idx, y+i, color);
				}
		}
		
		x = x+fwidth+fontSpace;
	}	

	cDispRstArea();							//reset the display area for 128*160 after DispSetPixel()
}

/*
*********************************************************************************************************
*                 CONVERT A STRING FROM DATA MEMORY (RAM) TO PIXEL DATA AND DISPLAY AT x,y
*
* Description : This function outputs a string in RAM in graphic mode (pixel-by-pixel) at a position
*				(x,y) following Cartesian coordinate system with variable font spacing def by fontSpace.
*				Character wrapping is allowed.
*				*** REQUIRE AT LEAST ONE FONT SET (sysfont.h) included under this file ****
*				Cartesian coordinates as (0,0) at the top left hand corner, (127,159) at the 
*				lower right hand corner, as seen from frontal up view of the PIC18-LF4550-STK1 board
*
* Arguments   : 'x'    		0....MAX_COL_PIXEL-1 is matrix position in horizontal direction (x-axis).
*							
*				'y'    		0....MAX_ROW_PIXEL-1 is matrix position in vertical direction (y-axis).
*
*				'*pStr'		pointer to an ASCII string in RAM, null terminated
*				'fontSpace' font spacing from 1 to 127
*				'color'		16-bit color possible
* Returns     : none
* Notes		  : 
* Example 	  : ....
*				char s[20];
*				main() 
*				{ 
*					cDispPixStrAt(0,20,s,1,RGB(31,0,0)); 	//Display the ASCII string in s[20]
*															//at (x,y)=(0,20), font space = 1
*															//color in red
*				}
*********************************************************************************************************
*/
void cDispStrAt(INT8U x, INT8U y, char *pStr, INT8U fontSpace, INT16U color)
{
	INT8U pixelByte,bit_idx, i;				//loop counters 
	INT8U c,fwidth, fheight;				//character, font width & height store

	while (*pStr)
	{
		c = *pStr++ - 32;
				
		fwidth = SYS_FNT[c].fontWidth;
		fheight = 8;

		if((x+fwidth)>MAX_COL_PIXEL) 		//character wrapping here				
		{
			x=0;
			y+=fheight;
		}

		if((y+fheight)>MAX_ROW_PIXEL)		//check for y boundary				
		{
			y=0;
		}

		for(i=0;i<fheight;i++)
		{
			pixelByte = SYS_FNT[c].fontBody[i];
			for(bit_idx=0; bit_idx<fwidth; bit_idx++)
				{
						if((pixelByte)&(0x80>>bit_idx))
							cDispSetPixel(x+bit_idx, y+i, color);
				}
		}
		
		x = x+fwidth+fontSpace;
	}	

	cDispRstArea();							//reset the display area for 128*160 after DispSetPixel()
}

/*
*********************************************************************************************************
*                         				WRITE A PATTERN AT X, Y COORDINATES
*
* Description : This function writes a particular pattern of size patWidth and patHeight at a position 
*				defined by (x,y) following Cartesian coordinate system. 
*				Pattern defined in ROM (flash)
*				Cartesian coordinates as (0,0) at the top left hand corner, (127,159) at the 
*				lower right hand corner, as seen from frontal up view of the PIC18-LF4550-STK1 board
* Arguments   : 'x'    		0....MAX_COL_PIXEL-1 is matrix position in horizontal direction (x-axis)
*				'y'    		0....MAX_ROW_PIXEL-1 is matrix position in vertical direction (y-axis)
*				'*pPat'		pointer to pattern structure in const rom (FLASH)
*				'color'  	color in 16-bit mode
* Returns     :	none
* Notes		  : Microchip C18 complier dependent as const rom keyword involved
*********************************************************************************************************
*/
void cDisprPatAt(INT8U x, INT8U y, const rom INT16U *pPat)
{
 	INT16U   col, row, pWidth, pHeight, pixel;

	pWidth  = pPat[0];
	pHeight = pPat[1];

	for(row=0;row<pHeight;row++)
		{	
			for(col=0;col<pWidth;col++)
			{
				pixel = pPat[col+row*pWidth+2];
				cDispSetPixel(col+x,row+y,pixel);
			}
		}	
	cDispRstArea();							//reset the display area for 128*160 after DispSetPixel()
}

/*
*********************************************************************************************************
*                  		ADJUST BRIGHTNESS & CONSTRAST IN NORMAL DISPLAY MODE
*
* Description : This function updates the brightness & contrast control value in normal display mode
* Arguments   : (INT8U) 'brightness'	0...255
*				(INT8U) 'contrast'  	0...63
* Returns     :	none
* Notes		  : 
*********************************************************************************************************
*/
void cDispSetQuali(INT8U brightness, INT8U contrast)
{
	//brightness control in normal display mode
	cDispWrCmd(CONTRAST_CONTROL1);	
	cDispWrCmd(brightness);
	//contrast control in normal display mode	
	cDispWrCmd(CONTRAST_CONTROL3);	
	cDispWrCmd(contrast);
		
}

/*
*********************************************************************************************************
*                         			LOW LEVEL PORT INITIALIZATION
*
* Description : This function performs low level port initialization
* Arguments   : none
* Returns     : none
* Notes		  : Hardware specific
*********************************************************************************************************
*/
void cDispInitPort(void)
{	
	cDISP_CS		= HIGH;			//de-select the color LCD to start with
	cDISP_A0		= HIGH;
	cDISP_WR		= HIGH;
	cDISP_RD		= HIGH;
	cDISP_DATA		= 0x00;
	cDISP_BL		= HIGH;			//backlight = OFF to start

	TRIS_cDISP_CS 	= OUTPUT;
	TRIS_cDISP_A0	= OUTPUT;
	TRIS_cDISP_WR	= OUTPUT;
	TRIS_cDISP_RD	= OUTPUT;
	TRIS_cDISP_DATA	= OUTPUT;
	TRIS_cDISP_BL	= OUTPUT;
}

/*
*********************************************************************************************************
*                         				LOW LEVEL COMMAND WRITE TO LCD
*
* Description : This function performs low level command write to LCD
* Arguments   : (INT8U) 'cmd' 	is the command written to the LCD module
* Returns     : none
* Notes		  : Hardware specific. Delay required for individual Fosc
*********************************************************************************************************
*/
void cDispWrCmd(INT8U cmd)
{
	cDISP_A0 	= LOW;			//A0 LOW for command
	cDISP_CS 	= LOW;	
	cDISP_DATA 	= cmd;	
	cDISP_WR 	= LOW;
	cDISP_WR 	= HIGH;															
	cDISP_CS 	= HIGH;
}
/*
*********************************************************************************************************
*                         				LOW LEVEL DATA WRITE TO LCD
*
* Description : This function performs low level display data write to LCD
* Arguments   : (INT8U) 'dat' 	is the data written to the LCD module
* Returns     : none
* Notes		  : Hardware specific. Delay required for individual Fosc
*********************************************************************************************************
*/
void cDispWrDat(INT8U dat)
{
	cDISP_A0 	= HIGH;		//A0 HIGH for data
	cDISP_CS 	= LOW;	
	cDISP_DATA 	= dat;	
	cDISP_WR 	= LOW;
	cDISP_WR 	= HIGH;															
	cDISP_CS 	= HIGH;
}

/*
*********************************************************************************************************
*                         					cDispBackLite
*
* Description : This function control the LCD backlight ENABLE -OR- DISABLE
* Arguments   : 'BOOLEAN' ctrl 		ENABLE 	for backlight ON
*									DISABLE	for backlight OFF
*			
* Returns     : none
* Notes		  : Hardware specific.
*********************************************************************************************************
*/
void cDispBackLite(BOOLEAN ctrl)
{
	(ctrl==ENABLE)?(cDISP_BL=LOW):(cDISP_BL=HIGH);
}



