/*
********************************************************************************************************************
*                              Digital Contrast and Brightness Adjustment for 65k color LCD
*
* Programmer 	: John Leung, TechToys. Hong Kong
* Web presence  : www.TechToys.com.hk
* Note			: This program demonstrates 5-way navigation joystick API functions & 65k color LCD API
*				  Click joystick up, down will tune the LCD brightness
*				  Click joystick left, right will tune the LCD contrast
*				  Click center up-down key will change the background color
* Language		: C18 complier version 2.40, MPLAB v7.60
* Hardware		: PIC18-LF4550-STK1 Rev 3b
* Date			: Version 1.0 (3rd August 2007)
* File			: main.c
********************************************************************************************************************
*/

#include <p18cxxx.h>
#include "delay.h"
#include "xKey.h"
#include "hdc1600.h"
#include "joystick.h"

/*
********************************************************************************************************************
*												Pragma Section
*
* Configuration bits under MPLAB v7.41 subject to changes on every little change on project settings.
* So #pragma config statments used for configuration bits instead of using Configuration bits tab in MPLAB
* Note: Refer to pdf document "mcc18\doc\MPLAB-C18-Addendum.pdf" under mcc18 installation directory for config
* 		variables available
********************************************************************************************************************
*/
#pragma config FOSC   = HS			// HS oscillator
#pragma config CPUDIV = OSC4_PLL6	// OSC postscaler of 4 => CPU clock at 5MHz (20MHz/4)
#pragma config PWRT   = ON			// PowerUp timer ON
#pragma config WDT    = OFF			// watchdog timer off
#pragma config LVP    = OFF			// low level voltage OFF
#pragma config BOR    = OFF			// brown out detect OFF
#pragma config DEBUG  = ON			// Debug ON
#pragma config VREGEN = OFF			// Off 3.3V internal regulator
#pragma config PBADEN = OFF 		// PORTB<4:0> as digital IO on reset
#pragma config MCLRE  = ON			// Enable MCLRE for reset function

/*
********************************************************************************************************************
*							UNSIGNED CHAR VALUE TO ASCII STRING WITH NULL TERMINATE
*
* This function perform a decimal value conversion to ascii string with null termination instead using sprintf
* function. Only hundred value limit of 255 has been assumed as unsigned char only hold 255 anyway.
********************************************************************************************************************
*/
void dec2ascii(unsigned char *s, unsigned char i)
{
	if(i<10)
		{s[0]=i+48; s[1]=0;}
	else if(i<100)
		{s[0]=(i/10)+48; s[1] = (i%10)+48; s[2] = 0;}
	else
		{s[0]=(i/100)+48; s[1]=((i%100)/10)+48; s[2]=(i%10)+48; s[3]=0;}
}

/*
********************************************************************************************************************
*														MAIN
*
* Click on the joystick up, down, left, right, and center to feel the contrast / brightness values on 
* different background color
********************************************************************************************************************
*/
void main(void)
{
		unsigned char s[4];							//string to hold the contrast/brightness value
		unsigned char keyCode, counter=0;
		unsigned char brightness=BRIGHTNESS, contrast=CONTRAST;
		unsigned int  background = RGB(31,0,0);

		ADCON1 = 0x0F;								//PORTA all digital operation

		cDispInit();	
		cDispBackLite(ENABLE);
		cDispClr(background);						//clear screen in red color
		cDisprPatAt(40, 80, &joystick[0]);

		cDisprStrAt(1, 10, "BRIGHTNESS:", 1, RGB(0,0,0));
		dec2ascii(&s[0],brightness);
		cDispStrAt(80,10, &s[0], 1, RGB(0,0,0));

		cDisprStrAt(1, 20, "CONTRAST:", 1, RGB(0,0,0));
		dec2ascii(&s[0],contrast);
		cDispStrAt(80,20, &s[0], 1, RGB(0,0,0));

	 	xKeyInit();									//initialize the keypad module before using it
		xKeyRptEn = TRUE;							//enable auo-repeat

		for(;;){
			DelayMs(xKEY_SCAN_TASK_DLY);	//debounce the keys
			xKeyScanTask();					//scan for key press
			if(xKeyHit())
			{
				keyCode = xKeyGetKey();
				switch (keyCode)
				{
				case xKEY_UP_FLAG:
					dec2ascii(&s[0],brightness);
					cDispStrAt(80,10, &s[0], 1, background);	//clear the present brightness		
					cDispSetQuali(brightness++, contrast);
					dec2ascii(&s[0],brightness);				//redraw the new brightness
					cDispStrAt(80,10, &s[0], 1, RGB(0,0,0));					
					break;
				case xKEY_DN_FLAG:
					dec2ascii(&s[0],brightness);
					cDispStrAt(80,10, &s[0], 1, background);	//clear the present brightness
					cDispSetQuali(brightness--, contrast); 
					dec2ascii(&s[0],brightness);				//redraw the new brightness
					cDispStrAt(80,10, &s[0], 1, RGB(0,0,0));
					break;
				case xKEY_LT_FLAG:
					dec2ascii(&s[0],contrast);
					cDispStrAt(80,20, &s[0], 1, background);	//clear the present contrast
					contrast = (contrast-1)%64;	
					cDispSetQuali(brightness, contrast);
					dec2ascii(&s[0],contrast);
					cDispStrAt(80,20, &s[0], 1, RGB(0,0,0));	//redraw the new contrast
					break;
				case xKEY_RT_FLAG:
					dec2ascii(&s[0],contrast);
					cDispStrAt(80,20, &s[0], 1, background);	//clear the present contrast
					contrast = (contrast + 1)%64;
					cDispSetQuali(brightness, contrast);
					dec2ascii(&s[0],contrast);
					cDispStrAt(80,20, &s[0], 1, RGB(0,0,0));	//redraw the new contrast
					break;
				case xKEY_CTR_FLAG:
					counter = (counter+1)%7;
					switch(counter)
					{
						case 0:	background = RGB(31,0,0);
								break;
						case 1: background = RGB(0,63,0);
								break;
						case 2: background = RGB(0,0,31);
								break;
						case 3: background = RGB(31,63,0);
								break;
						case 4: background = RGB(0,63,31);
								break;
						case 5: background = RGB(31,0,31);
								break;
						case 6: background = RGB(31,63,31);	
					}
					cDispClr(background);						//clear screen in background color
					cDisprPatAt(40, 80, &joystick[0]);
					cDisprStrAt(1, 10, "BRIGHTNESS:", 1, RGB(0,0,0));
					dec2ascii(&s[0],brightness);
					cDispStrAt(80,10, &s[0], 1, background);	//clear the present brightness
					dec2ascii(&s[0],brightness);
					cDispStrAt(80,10, &s[0], 1, RGB(0,0,0));

					cDisprStrAt(1, 20, "CONTRAST:", 1, RGB(0,0,0));
					dec2ascii(&s[0],contrast);
					cDispStrAt(80,20, &s[0], 1, background);	//clear the present contrast
					dec2ascii(&s[0],contrast);
					cDispStrAt(80,20, &s[0], 1, RGB(0,0,0));
				}
			}
		}
}